/*-------------------------------------------------------------------------------
  AudioGrabberReal256.cpp:   A Teensy Audio Library compatible object to extract
                             a 256 length buffer of real data from the audio stream.
  
  Author:    Derek Rowell
  Modified:  October 2, 2019

  Function:  Retrieve a buffer of 256 real samples from the Teensy audio data
             stream for external processing.  The data is returned in Q15.1
             format (int16_t).
             
  Usage:     if(myGrabber.newDataAvailable) {
               myGrabber.grab(myBuffer);
             }
             where myGrabber has been declared as an instance of the class
             AudioGrabberReal256 and myBuffer is a length 256 int16_t buffer.
  ---
  Copyright (c) 2019 Derek Rowell
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
 --------------------------------------------------------------------------------*/

#include "AudioGrabberReal256.h"

static void copy_to_buffer(void *destination, const void *source) {
  const int16_t *src = (const int16_t *)source;
  int16_t *dst = (int16_t *)destination;
  for (int i=0; i<AUDIO_BLOCK_SAMPLES; i++) *dst++ = *src++;  
}

//--------------------------------------------------------------------------------
void AudioGrabberReal256::update(void) { 
  audio_block_t *blockIn;
  blockIn = receiveReadOnly(0);
  if (!blockIn)  return;
  // Organize the input buffer
  copy_to_buffer(_buffer+_buffStart, blockIn->data);       // move previous data block down
  _buffStart = (_buffStart+128)%256;
  if (_buffStart == 0) {
    // Wait for any current "grab" to complete
    while(_transferringData){};
    // Transfer the new data block to the output buffer
    _dataBufferValid    = false;
    for (int i=0; i<256; i++) {
      _outBuffer[i] = _buffer[i];
    }
    _newDataIsAvailable = true;
    _dataBufferValid    = true;
  }
  release(blockIn);
}
//
//--------------------------------------------------------------------------------
  bool AudioGrabberReal256::newDataAvailable(void) {
    return _newDataIsAvailable; 
  }
//
//--------------------------------------------------------------------------------
  void AudioGrabberReal256::grab(int16_t *destination) {
     if (_dataBufferValid){
      _transferringData = true;
      for (int i=0; i<BUFF_LENGTH; i++) *destination++ = _outBuffer[i];
    }
    _transferringData   = false;
    _newDataIsAvailable = false;     
  }
